/*
 * Copyright (C) 2022 Associated Universities, Inc. Washington DC, USA.
 *
 * This file is part of NRAO Workspaces.
 *
 * Workspaces is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Workspaces is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
 */

package db

import (
	"database/sql"
	"fmt"
	_ "github.com/lib/pq"
)

type UserInfo struct {
	Name          string
	Email         string
	Group         string
	IsUnavailable bool
	IsAvailable   bool
}

/**
 * Check if an error happened and panic on it if so
 *
 * @param err An error object to report
 **/
func checkError(err error) {
	if err != nil {
		panic(err)
	}
}

//AddUser
/**
 * Add a user to the qa_staff table in the database
 *
 * @param user a UserInfo struct instance with the name, email, and stage of the person to be added
 * @param db_info a DbInfo type with information to connect to the database
 **/
func AddUser(user UserInfo, connectionInfo DbInfo) {
	// Get a connection to the database
	db := GetConnection(connectionInfo)

	defer func(db *sql.DB) {
		err := db.Close()
		checkError(err)
	}(db)

	// Add the user, new users are assumed to always be available
	insertStatement := `insert into "qa_staff"("user_name", "group", "available", "email") values($1, $2, $3, $4)`
	_, err := db.Exec(insertStatement, user.Name, user.Group, true, user.Email)
	checkError(err)

	fmt.Println("Updated " + user.Name + " to qa_staff")
}

//UpdateUser
/**
* Update existing user in the qa_staff table
 */
func UpdateUser(user UserInfo, connectionInfo DbInfo) {
	// Get a connection to the database
	db := GetConnection(connectionInfo)

	defer func(db *sql.DB) {
		err := db.Close()
		checkError(err)
	}(db)

	// update the user
	conditions := buildUpdateConditions(user)
	updateStatement := `update qa_staff set ` + conditions + ` where user_name=$1`

	_, err := db.Exec(updateStatement, user.Name)
	checkError(err)

	fmt.Println("Updated " + user.Name)
}

func buildUpdateConditions(user UserInfo) string {
	// possible updatable field are email and available, changes in user groups should be new table entries
	updateEmail := user.Email
	updateUnavailability := user.IsUnavailable
	updateAvailability := user.IsAvailable
	//always set availability since we can't tell if it's actually being changed.
	setCondition := ""

	if updateEmail != "" {
		setCondition = "email='" + user.Email + "'"
	}
	if updateUnavailability == true {
		//user is now unavailable

		if len(setCondition) > 0 {
			// we are updating multiple fields
			setCondition = setCondition + ", "
		}
		setCondition = setCondition + "available=false"
	}
	if updateAvailability == true {
		//user is now available

		if len(setCondition) > 0 {
			// we are updating multiple fields
			setCondition = setCondition + ", "
		}
		setCondition = setCondition + "available=true"
	}
	return setCondition
}

//RemoveUser
/**
 * Remove a user from the qa_staff table in the database
 *
 * @param user a UserInfo struct instance with the information of the person to be removed
 * @param db_info a DbInfo type with information to connect to the database
 **/
func RemoveUser(user UserInfo, connectionInfo DbInfo) {
	// Get a connection to the database
	db := GetConnection(connectionInfo)

	defer func(db *sql.DB) {
		err := db.Close()
		checkError(err)
	}(db)

	// Remove the user
	deleteStatement := `delete from "qa_staff" where "user_name"=$1 and "group"=$2`
	_, err := db.Exec(deleteStatement, user.Name, user.Group)
	checkError(err)

	fmt.Println("Removed " + user.Name + ",group " + user.Group + " from qa_staff")
}
